require 'src/hierarchy'
require 'src/config'

require 'fox'
require 'thread'

include Fox

FILENAME="rfind.data"

class FindWindow < FXMainWindow
	def initialize(app)
		super(app, "Browse", nil, nil, DECOR_ALL, 0, 0, 350, 600)
		@config = Config.new
		buildGui
	end
	
	def buildGui
		# Contents 
		@mainFrame = FXVerticalFrame.new(self, FRAME_NONE|LAYOUT_FILL_X|LAYOUT_FILL_Y)
		
		# Switcher
		@tabbook = FXTabBook.new(@mainFrame, nil, 0, LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_RIGHT)

		# first tab item	
		@tab1 = FXTabItem.new(@tabbook, "&Search", nil)		
		@searchFrame = FXVerticalFrame.new(@tabbook, 
			FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y)
		
		# top query frame: "Search [_________] [..]"
		@queryText = FXTextField.new(@searchFrame, 20, nil, 0, 
			LAYOUT_FILL_X|FRAME_SUNKEN|LAYOUT_SIDE_TOP)
		@queryText.connect(SEL_CHANGED) do onQueryTextChange end
				
		# Tree view
		@tree = FXTreeList.new(@searchFrame, 20, nil, 0,
			FRAME_SUNKEN|TREELIST_BROWSESELECT|TREELIST_SHOWS_LINES|TREELIST_SHOWS_BOXES|
			TREELIST_ROOT_BOXES|LAYOUT_FILL_X|LAYOUT_FILL_Y)
		@tree.connect(SEL_CHANGED) do |s, sel, data| onTreeClick(data) end
		
		# status frame
		@statusFrame = FXHorizontalFrame.new(@mainFrame,
			FRAME_SUNKEN|LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0)
		@statusBar = FXLabel.new(@statusFrame, "Loading...\tDate of last scan | scanned directory | number of files")
		
		
		# Config tab #############################
		@tab2 = FXTabItem.new(@tabbook, "&Config", nil)
		@configFrame = FXVerticalFrame.new(@tabbook,
			FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y)
		
		# Directory
		FXLabel.new(@configFrame, "Directory")
		
		# Source directory button
		@directoryFrame = FXHorizontalFrame.new(@configFrame)
		
		@sourceDirectoryBtn = FXButton.new(@directoryFrame, "...")
		@sourceDirectoryBtn.connect(SEL_COMMAND) do onSourceDirectoryBtnClick end
		
		@directoryStatusText = FXLabel.new(@directoryFrame, "nothing indexed")

		# Rules
		FXLabel.new(@configFrame, "Rules")
		
		# Rules textbox
		@rulesText = FXText.new(@configFrame, nil, 0, 
			LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|LAYOUT_SIDE_TOP)
		@rulesText.connect(SEL_CHANGED) do onRulesTextChange end
	end	
		
	def create
		super
		show(PLACEMENT_SCREEN)
		loadSaved
	end
	
	def loadSaved
		threaded do
			guiNotIndexed
			if File.exists? FILENAME
				@hier = HierarchySearcher.load(FILENAME)
				@sourceDirectoryBtn.text = @hier.baseDir if @hier
				@statusBar.text = "#{@hier.size} files indexed."
				@directoryStatusText.text = @hier.date.strftime "indexed on %Y/%m/%d, %H:%M" if @hier
				guiIndexed
			end
		end
		
		# load config
		if File.exists? "rfind.conf"
			File.open("rfind.conf", "r") do |f|
				@rulesText.text = f.read
				@config.rules = @rulesText.text
			end
		end
	end
	
	def threaded
		if @updateThread != nil
			# A thread has to check @stopThread and exist as soon as possible
			@stopThread = true
			@updateThread.join
		end
		@stopThread = false
		@updateThread = Thread.new(self) do
			yield
		end
	end
	
	def onSourceDirectoryBtnClick	
		dialog = FXDirDialog.new(self, "Choose directory to index")
		if dialog.execute  != 0
			threaded do
				@statusBar.text = "Indexing, please wait..."
				@queryText.enabled = false
				@rulesText.enabled = false
				@sourceDirectoryBtn.text = "#{dialog.directory}"
				@hier = HierarchySearcher.new(dialog.directory) do |entry, size|
					@statusBar.text = "#{size} files indexed." if size % 1000 == 0 
					@directoryStatusText.text = "currently indexing #{size} files"  if size % 1000 == 0 
					Thread.exit if @stopThread
					entry.include?(".svn") ||
					entry.include?("System Volume Information")
				end
				@statusBar.text = "Saving..."
				@hier.save(FILENAME)
				guiIndexed
			end
		end
	end

	def onTreeClick(data)
		# rebuild full path
		hier = Array.new
		while data
			hier.push data.text
			data = data.parent
		end
		hier.push @hier.baseDir
		path = hier.reverse!.join("/")
		path.gsub!("//", "/")
		
		# do something with full path
		command = @config.match(path)
		if command
			Thread.new do system(command) end
		end
	end
	
			
	def onQueryTextChange
		threaded do
			reg = nil
			begin
				reg = Regexp.new(@queryText.text, Regexp::IGNORECASE)
			rescue
				reg = ""
			end
			@tree.clearItems
			
			count = 0
			@hier.findHierarchy(reg) do |name, parent, isResult|
				Thread.exit if @stopThread
				item = @tree.addItemLast(parent, name)
				item.enabled = isResult
				count += 1 if isResult
				@statusBar.text = "#{count} matches out of #{@hier.size} entries"
				item.expanded = true
				item
			end
			@statusBar.text = "#{count} matches out of #{@hier.size} entries"
		end
	end
	
	def onRulesTextChange
		@config.rules = @rulesText.text
		# save rules
		File.open("rfind.conf", "w") do |f|
			f.write(@config.rules)
		end
	end

	def guiNotIndexed	
		@statusBar.text = "No files indexed."
		@directoryStatusText.text = "nothing indexed"
		@queryText.text = "You need to index a directory on the config tab"
		@queryText.enabled = false
	end
	
	def guiIndexed
		@queryText.enabled = true
		@queryText.text = ""
		@rulesText.enabled = true
		@statusBar.text = "#{@hier.size} files indexed."
		@directoryStatusText.text = @hier.date.strftime "indexed on %Y/%m/%d, %H:%M" if @hier
	end
end


if __FILE__ == $0
	application = FXApp.new("RFind", "RFind")
	FindWindow.new(application)
	application.create
	application.run
end

